part of fields.view;

class FieldSetView extends DivElement {
  
  static const TAG = 'fieldset-view';
  
  final _cache = new Map<Field, TableRowElement>.identity();
  Element _tbody;
  
  factory FieldSetView() => new Element.tag('div', TAG);
  
  FieldSetView.created() : super.created() {
    children(this, () {
      table(() => _tbody = tbody());
    });
  }

  void clear() {
    _tbody.children.clear();
    _cache.clear();
  }
  
  void refresh(Set<Field> fields) {
    for (Field f in _cache.keys.where((f) => 
        !fields.any((e) => identical(e, f)))) {
      _tbody.children.remove(_cache[f]);
    }
    for (int i = 0; i < fields.length; i++) {
      final Field f = fields.elementAt(i);        
      if (_cache.containsKey(f)) {
        final Element el = _cache[f];
        _insertChildAt(el, i);
        final FieldView fv = el.children[1].firstChild;
        fv.refresh();
      } else {
        final el = tr(() {
          td(() => span()..text = f.displayName)
          ..title = f.name
          ..classes = ['namebox'];
          td(() => append(new FieldView(f)));
        });
        _cache[f] = el;
        _insertChildAt(el, i);
      }
    }
  }
  
  void _insertChildAt(Element element, int index) {
    if (index < _tbody.children.length) {
      _tbody.insertBefore(element, _tbody.children[index]);
    } else {
      _tbody.append(element);
    }
  }
}
