@TestGroup('Field')
library fields.test.model.field;

import 'package:bench/bench.dart';
import 'package:unittest/unittest.dart' hide Description;
import 'package:fields/model.dart';

main() => reflectTests();

// -----------------------------------------------------------------------------
// Setup
// -----------------------------------------------------------------------------

HasFields hasFields;
@Setup
void setup() {
  hasFields = new HasFields();
}

class HasFields {
  int i = 7;
  int get ri => 42;
  num n = 3.14;
  final num rn = 6.28;
  double d = 1024.4201;
  final double rd = 1.024;
  String s = 'snarf';
  final String rs = 'immutable';
  List li = [ 1, 2, 3, 4];    
  Map m = { 'a' : 1, 'b' : 2 };
}

// -----------------------------------------------------------------------------
// Synthetic
// -----------------------------------------------------------------------------

@Test('Test the construction of a synthetic int field.')
void testInt() {  
  final f = new Field<int>('si', () => hasFields.i, 
      setter: (int i) => hasFields.i = i);  
  expect(f.value, equals(7));
  expect(f.name, equals('si'));
  expect(f.type, equals(int));
  expect(f.isReadOnly, isFalse);
  expect(f.isPrivate, isFalse);
  expect(f.children, isEmpty);
  expect(f.annotations, isEmpty);
}

@Test('Test the construction of a synthetic read-only int field.')
void testIntReadOnly() {
  final f = new Field<int>('sri', () => hasFields.ri);
  expect(f.value, equals(42));    
  expect(f.name, equals('sri'));
  expect(f.type, equals(int));
  expect(f.isReadOnly, isTrue);
  expect(f.isPrivate, isFalse);
  expect(f.children, isEmpty);
  expect(f.annotations, isEmpty);
}

@Test('Test that setting a value on a synthetic read-only field throws.')
@ExpectError(isAssertionError)
void testSyntheticSetValueReadOnlyThrows() {
  final f = new Field<int>('sri', () => hasFields.ri);
  f.value = 13;
}

@Test('Test that setting a value on a sythetic field adds to the stream.')
void testSyntheticSetValue() {
  final f = new Field<int>('si', () => hasFields.i, 
      setter: (int i) => hasFields.i = i);
  f.onValueChange.listen(expectAsync((v) {
    expect(v, equals(8));
  }));  
  f.value = 8;
  expect(f.value, equals(8));
  expect(hasFields.i, equals(8));
}

@Test('Test getting and setting the value of a synthetic field using `call`')
void testCall() {
  final f = new Field<int>('si', () => hasFields.i, 
        setter: (int i) => hasFields.i = i);  
  expect(f(), equals(7));
  expect(f(42), equals(42));  
  expect(f(), equals(42));
}
